/* 
   Title : This is the template class for settings a document expiry date.
   Date  : 14/01/2003
   Author: Olli Aro
		   Clicks and Links Ltd
*/

package com.opencms.modules.expirydate;

import com.opencms.core.*;
import com.opencms.file.*;
import com.opencms.template.*;
import com.opencms.template.cache.*;
import com.opencms.util.*;
import com.opencms.workplace.*;
import java.io.*;
import java.net.*;
import javax.servlet.http.*;
import java.util.*;
 
public class ExpiryDate extends ModuleVariables
{	
	public static final String C_EXPIRY_DATE="expiry_date";
	public static final String C_START_DATE="start_date";
	
	// Get the possible expiry date from the page parameters.
	
	public String checkExpiryDate(CmsObject cms, String tagcontent, A_CmsXmlContent doc, Object userObject)
	throws CmsException 
	{
		String error="";
		String requestedUri=cms.getRequestContext().getUri();
		// Get the possible start date from the page parameters.
		String startDate=cms.readProperty(requestedUri,C_START_DATE);
	
		//If start date was set.
		if((startDate!=null)&&(!startDate.equals("")))
		{
			StringTokenizer st=new StringTokenizer(startDate,".");
			
			//If the start date attribute had been set properly. 
			if(st.countTokens()==3)
			{
				//Put the string in a date format.
				String day="";
				String month="";
				String year="";
				if (st.hasMoreTokens())
				    day=st.nextToken();
				if (st.hasMoreTokens())
				    month=st.nextToken();
				if (st.hasMoreTokens())
				    year=st.nextToken();
				Calendar cal=Calendar.getInstance();
				cal.set(Calendar.MILLISECOND,0); 
				cal.set(Calendar.SECOND,0); 
				cal.set(Calendar.MINUTE,0);
				cal.set(Calendar.HOUR,0); 
				cal.set(Calendar.DAY_OF_MONTH,(new Integer(day)).intValue());
				cal.set(Calendar.MONTH,(new Integer(month)).intValue()-1);
				cal.set(Calendar.YEAR,(new Integer(year)).intValue());
				Date startDateD=cal.getTime();
				Date currentDate=new Date(); 
			
				//If the current date is less or 
				//equal than the start date, then redirect to the redirection page.
				if(currentDate.getTime()<startDateD.getTime())
				{
					//Get the url to the data expiry page.
					HttpServletResponse response=(HttpServletResponse)cms.getRequestContext().getResponse().getOriginalResponse();    	
					
					//Redirect to the expiry url defined as a method parameter.
					try
					{
						//response.sendRedirect(tagcontent);
						response.sendError(response.SC_NOT_FOUND);    
					}catch(Exception e){error=e.getMessage();}
				}
			}
		}
		
		// Get the possible expiry from the page parameters.
		String expiryDate=cms.readProperty(requestedUri,C_EXPIRY_DATE);
	
		//If expiry date was set.
		if((expiryDate!=null)&&(!expiryDate.equals("")))
		{
			StringTokenizer st=new StringTokenizer(expiryDate,".");
			
			//If the expiry date attribute had been set properly. 
			if(st.countTokens()==3)
			{
				//Put the string in a date format.
				String day="";
				String month="";
				String year="";
				if (st.hasMoreTokens())
				    day=st.nextToken();
				if (st.hasMoreTokens())
				    month=st.nextToken();
				if (st.hasMoreTokens())
				    year=st.nextToken();
				Calendar cal=Calendar.getInstance();
				cal.set(Calendar.MILLISECOND,0); 
				cal.set(Calendar.SECOND,0); 
				cal.set(Calendar.MINUTE,0);
				cal.set(Calendar.HOUR,0); 
				cal.set(Calendar.DAY_OF_MONTH,(new Integer(day)).intValue());
				cal.set(Calendar.MONTH,(new Integer(month)).intValue()-1);
				cal.set(Calendar.YEAR,(new Integer(year)).intValue());
				Date expiryDateD=cal.getTime();
				Date currentDate=new Date(); 
			
				//If the current date is greater or 
				//equal than the expiry date, then redirect to the redirection page.
				if(currentDate.getTime()>expiryDateD.getTime())
				{
					//Get the url to the data expiry page.
					HttpServletResponse response=(HttpServletResponse)cms.getRequestContext().getResponse().getOriginalResponse();    	
					
					//Redirect to the expiry url defined as a method parameter.
					try
					{
						//response.sendRedirect(tagcontent);
						response.sendError(response.SC_NOT_FOUND);    
					}catch(Exception e){error=e.getMessage();}
				}
			}
		}
		
		return error;
	}

	/**
	* This method checks the "expiry_date" property of a page and returns a boolean to indicate whether it has expired or not.
	* 
	* @param cms CmsObject object for accessing system resources.
	* @param tagContent Unused in this special case of a user method. Can be ignored.
	* @param doc Reference to the A_CmsXmlContent object of the initiating XLM document.
	* @param userObject Hashtable with parameters.
	* @return A string of HTML containing a line from the data file optionally surrounded by HTML tags.
	* @exception CmsException
	**/	
	protected static boolean hasExpired(CmsObject cms,String resURI)
	{
		boolean hasExpired=false;
		
		try{
			// Get the possible start date from the page parameters.
			String startDate=cms.readProperty(resURI,C_START_DATE);
		
			//If start date was set.
			if((startDate!=null)&&(!startDate.equals("")))
			{
				StringTokenizer st=new StringTokenizer(startDate,".");
				
				//If the start date attribute had been set properly. 
				if(st.countTokens()==3)
				{
					//Put the string in a date format.
					String day="";
					String month="";
					String year="";
					if (st.hasMoreTokens())
					    day=st.nextToken();
					if (st.hasMoreTokens())
					    month=st.nextToken();
					if (st.hasMoreTokens())
					    year=st.nextToken();
					Calendar cal=Calendar.getInstance();
					cal.set(Calendar.MILLISECOND,0); 
					cal.set(Calendar.SECOND,0); 
					cal.set(Calendar.MINUTE,0);
					cal.set(Calendar.HOUR,0); 
					cal.set(Calendar.DAY_OF_MONTH,(new Integer(day)).intValue());
					cal.set(Calendar.MONTH,(new Integer(month)).intValue()-1);
					cal.set(Calendar.YEAR,(new Integer(year)).intValue());
					Date startDateD=cal.getTime();
					Date currentDate=new Date(); 
				
					//If the current date is less or 
					//equal than the start date, then the content has expired.
					if(currentDate.getTime()<startDateD.getTime())
						hasExpired=true;
				}
			}
		
			// Get the possible expiry from the page parameters.
			String expiryDate=cms.readProperty(resURI,C_EXPIRY_DATE);
	
			//If expiry date was set.
			if((expiryDate!=null)&&(!expiryDate.equals("")))
			{
				StringTokenizer st=new StringTokenizer(expiryDate,".");
				
				//If the expiry date attribute had been set properly. 
				if(st.countTokens()==3)
				{
					//Put the string in a date format.
					String day="";
					String month="";
					String year="";
					if (st.hasMoreTokens())
					    day=st.nextToken();
					if (st.hasMoreTokens())
					    month=st.nextToken();
					if (st.hasMoreTokens())
					    year=st.nextToken();
					Calendar cal=Calendar.getInstance();
					cal.set(Calendar.MILLISECOND,0); 
					cal.set(Calendar.SECOND,0); 
					cal.set(Calendar.MINUTE,0);
					cal.set(Calendar.HOUR,0); 
					cal.set(Calendar.DAY_OF_MONTH,(new Integer(day)).intValue());
					cal.set(Calendar.MONTH,(new Integer(month)).intValue()-1);
					cal.set(Calendar.YEAR,(new Integer(year)).intValue());
					Date expiryDateD=cal.getTime();
					Date currentDate=new Date(); 
				
					//If the current date is greater or 
					//equal than the expiry date, then the resource has expired.
					if(currentDate.getTime()>expiryDateD.getTime())
						hasExpired=true;
				}
			}
		}catch(CmsException e){}
		return hasExpired;
	}
	
	// Set caching	false.
	public CmsCacheDirectives getCacheDirectives(CmsObject cms,String templateFile,String elementName,Hashtable parameters, String templateSelector) 
	{
		CmsCacheDirectives cd=new CmsCacheDirectives(false); 
		cd.setCacheUser(true); 
		cd.setCacheUri(true);
		return cd;
	}
}
