/* 
   Title : This is a navigation template extension class.
   Date  : 23/01/2003
   Author: Olli Aro
		   Clicks and Links Ltd
*/

package com.opencms.modules.expirydate;

import com.opencms.defaults.*;
import com.opencms.file.*;
import com.opencms.core.*;
import com.opencms.util.*;
import com.opencms.template.*;
import java.util.*;

public class NavTemplateExtension extends CmsXmlNav 
{	
	/**
     * gets the navigation of current folder.
     *
     * @param cms CmsObject Object for accessing system resources.
     * @param tagcontent Unused in this special case of a user method. Can be ignored.
     * @param doc Reference to the A_CmsXmlContent object of the initiating XLM document.
     * @param userObj Hashtable with parameters.
     * @return byte[] with the content of this subelement.
     * @throws CmsException
     */
    public Object getNavCurrent(CmsObject cms, String tagcontent, A_CmsXmlContent doc, Object userObject)
            throws CmsException {
        // template file
        CmsXmlTemplateFile template=(CmsXmlTemplateFile)doc;
        // check whether there exist entry datablock
        if (!template.hasData("naventry")) {
            return "".getBytes();
        }
        String currentFolder=cms.getRequestContext().currentFolder().getAbsolutePath();
        int level=extractLevel(cms,currentFolder)+1;
        String exact="false";
        // tagcontent determines the folder starting from root folder.
        // if tagcontent is null, then the navigation of root folder must be showed.
        if (!tagcontent.equals("")) {
            try {
                if (tagcontent.indexOf(",")!=-1) {
                    level=Integer.parseInt(tagcontent.substring(0,tagcontent.indexOf(",")));
                    exact=tagcontent.substring(tagcontent.indexOf(",")+1).toLowerCase();
                } else {
                    level=Integer.parseInt(tagcontent);
                }
            } catch(NumberFormatException e) {
                level=extractLevel(cms,currentFolder)+1;
                exact=tagcontent.toLowerCase();
                if (!exact.equals("true")) {
                    exact="false";
                }
            }
        }
        // get the folder of level
        currentFolder="/";
        StringTokenizer st = new StringTokenizer(cms.getRequestContext().currentFolder().getAbsolutePath(),"/");
        int count=st.countTokens()+1;
        if (exact.equals("true") && level!=count) {
            return "".getBytes();
        }
		while (st.hasMoreTokens()) {
            if (level>1) {
                currentFolder=currentFolder+st.nextToken()+"/";
                level--;
            } else {
                break;
            }
        }
        // register this folder for changes
        Vector vfsDeps = new Vector();
        vfsDeps.add(cms.readFolder(currentFolder));
        registerVariantDeps(cms, doc.getAbsoluteFilename(), null, null,
                        (Hashtable)userObject, vfsDeps, null, null);
        // get all resources in current folder
        Vector resources=cms.getSubFolders(currentFolder);
        Vector allFile=cms.getFilesInFolder(currentFolder);
        resources.ensureCapacity(resources.size() + allFile.size());
        Enumeration e = allFile.elements();
        while (e.hasMoreElements()) {
            resources.addElement(e.nextElement());
        }
        // if there is not exist current datablock then take the entry datablock
        if (!template.hasData("navcurrent")) {
            template.setData("navcurrent", template.getData("naventry"));
        }
        return buildNav(cms,doc,userObject,resources).getBytes();
    }
	
	/**
     * gets the navigation of root folder or parent folder starting from root folder.
     *
     * @param cms CmsObject Object for accessing system resources.
     * @param tagcontent Unused in this special case of a user method. Can be ignored.
     * @param doc Reference to the A_CmsXmlContent object of the initiating XLM document.
     * @param userObj Hashtable with parameters.
     * @return byte[] with the content of this subelement.
     * @throws CmsException
     */
    public Object getNavRoot(CmsObject cms, String tagcontent, A_CmsXmlContent doc, Object userObject)
            throws CmsException {

        // template file
        CmsXmlTemplateFile template=(CmsXmlTemplateFile)doc;
        // check whether there exist entry datablock
        if (!template.hasData("naventry")) {
            return "".getBytes();
        }
        int level=1;
        String exact="false";
        // tagcontent determines the folder starting from root folder.
        // if tagcontent is null, then the navigation of root folder must be showed.
        if (!tagcontent.equals("")) {
            try {
                if (tagcontent.indexOf(",")!=-1) {
                    level=Integer.parseInt(tagcontent.substring(0,tagcontent.indexOf(",")));
                    exact=tagcontent.substring(tagcontent.indexOf(",")+1).toLowerCase();
                } else {
                    level=Integer.parseInt(tagcontent);
                }
            } catch(NumberFormatException e) {
                level=1;
                exact=tagcontent.toLowerCase();
                if (!exact.equals("true")) {
                    exact="false";
                }
            }
        }
        String currentFolder=extractFolder(cms,level,exact);
        if (currentFolder.equals(""))
            return "".getBytes();
        // register this folder for changes
        Vector vfsDeps = new Vector();
        vfsDeps.add(cms.readFolder(currentFolder));
        registerVariantDeps(cms, doc.getAbsoluteFilename(), null, null,
                        (Hashtable)userObject, vfsDeps, null, null);
        // get all resources, it means all files and folders.
        Vector resources=cms.getSubFolders(currentFolder);
        Vector allFile=cms.getFilesInFolder(currentFolder);
        resources.ensureCapacity(resources.size() + allFile.size());
        Enumeration e = allFile.elements();
        while (e.hasMoreElements()) {
            resources.addElement(e.nextElement());
        }
        // if there is not exist current datablock then take the entry datablock
        if (!template.hasData("navcurrent")) {
            template.setData("navcurrent", template.getData("naventry"));
        }
        return buildNav(cms,doc,userObject,resources).getBytes();
    }
	
/**
     * Builds the navigation.
     *
     * @param cms CmsObject Object for accessing system resources.
     * @param doc Reference to the A_CmsXmlContent object of the initiating XLM document.
     * @param resources a vector that contains the elements of navigation.
     * @param userObj Hashtable with parameters.
     * @return String that contains the navigation.
     */
    protected String buildNav(CmsObject cms, A_CmsXmlContent doc,Object userObject, Vector resources)
        throws CmsException {

        // get uri, currentfolder,servletpath and template file
        String requestedUri = cms.getRequestContext().getUri();
        // String currentFolder=cms.getRequestContext().currentFolder().getAbsolutePath();
        // String servletPath = cms.getRequestContext().getRequest().getServletUrl();
        CmsXmlTemplateFile template=(CmsXmlTemplateFile)doc;
        StringBuffer result = new StringBuffer();
        int size = resources.size();
		
        String navLink[] = new String[size];
        String navText[] = new String[size];
        float navPos[] = new float[size];
        // extract the navigation according to navigation position and nav text
        int max=extractNav(cms,resources,navLink,navText,navPos);
        // The arrays folderNames and folderTitles now contain all folders
        // that should appear in the nav.
        // Loop through all folders and generate output
        for(int i=0; i<max; i++) {
			
	//Do if the content has not expired.
	if(!ExpiryDate.hasExpired(cms,navLink[i]))
		{
				template.setData("navtext", navText[i]);
				template.setData("navcount", new Integer(i+1).toString());
				template.setData("navlevel", new Integer(extractLevel(cms,navLink[i])).toString());
				// check whether it is a folder or file
				String link="";
				if (navLink[i].endsWith("/")) {
				    // read the property of folder to find the link file.
				    String navIndex=cms.readProperty(navLink[i],C_PROPERTY_NAVINDEX);
				    // if there is not defined a property then take C_NAVINDEX constant
				    if (navIndex==null) {
				        navIndex=C_NAVINDEX;
				    }
				    try {
				        cms.readFile(navLink[i] + navIndex);
				        link=navLink[i] + navIndex;
				        template.setData("navlink", LinkSubstitution.getLinkSubstitution(cms, navLink[i] + navIndex));
				    } catch (CmsException e) {
				        template.setData("navlink", LinkSubstitution.getLinkSubstitution(cms, requestedUri));
				    }
				} else {
				    try {
				        cms.readFile(navLink[i]);
				        link=navLink[i];
				        template.setData("navlink", LinkSubstitution.getLinkSubstitution(cms, navLink[i]));
				    } catch (CmsException e) {
				        template.setData("navlink", LinkSubstitution.getLinkSubstitution(cms, requestedUri));
				    }
				}
				// Check if nav is current nav
				//if (navLink[i].equals(currentFolder) || navLink[i].equals(requestedUri)) {
				if (link.equals(requestedUri)) {
				    result.append(template.getProcessedDataValue("navcurrent", this, userObject));
				} else {
				    result.append(template.getProcessedDataValue("naventry", this, userObject));
				}
		}
        }
        return result.toString();
    }
}
